#!/usr/bin/env node --no-deprecation
"use strict";
// tslint:disable:no-console
Object.defineProperty(exports, "__esModule", { value: true });
require("babel-polyfill");
require("source-map-support/register");
const program = require("commander");
const fs = require("fs");
const git = require("isomorphic-git");
const process = require("process");
const assets_1 = require("./asset/assets");
const box_1 = require("./box/box");
const compiler_1 = require("./compile/compiler");
const deployer_1 = require("./deploy/deployer");
const invoker_1 = require("./invoke/invoker");
const tester_1 = require("./test/tester");
const tooler_1 = require("./tool/tooler");
const cliUtils_1 = require("./utils/cliUtils");
const patchCommander_1 = require("./utils/patchCommander");
const version_1 = require("./utils/version");
const walletManager_1 = require("./wallet/walletManager");
patchCommander_1.patchCommander(program);
git.plugins.set('fs', fs);
program
    .name('punica-ts')
    .description('Punica CLI - The Ontology Blockchain dApp development framework')
    .version(version_1.version, '-v, --version')
    .option('-p, --project [PATH]', 'specify a punica project directory')
    .option('-d, --debug', 'print exceptions');
program
    .command('init')
    .description('initialize new and empty Ontology DApp project')
    .action(() => {
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const box = new box_1.Box();
        return box.init(cliUtils_1.getProjectDir(program));
    });
});
program
    .command('unbox')
    .description('download a Punica Box, a pre-built Ontology DApp project')
    .option('--name <NAME>', 'specify which box to unbox')
    .action((options) => {
    const name = options.name;
    cliUtils_1.checkRequiredOption('name', name);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const box = new box_1.Box();
        return box.unbox(name, cliUtils_1.getProjectDir(program));
    });
});
program
    .command('boxes')
    .description('list all available boxes')
    .action(() => {
    return cliUtils_1.wrapDebug(program.debug, async () => {
        console.log('Looking for available boxes...');
        const box = new box_1.Box();
        const boxes = await box.list();
        console.log('Boxes:');
        boxes.forEach((b) => console.log(b));
    });
});
program
    .command('compile')
    .description('compile the specified contracts to avm and abi files')
    .option('--contracts [CONTRACTS]', 'specify contracts files in contracts dir')
    .option('--v2', 'use python 2.0 compiler')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        console.log('Compiling...');
        const compiler = new compiler_1.Compiler();
        await compiler.compile(projectDir, options.contracts, options.v2);
        console.log('Compiled, Thank you.');
    });
});
program
    .command('deploy')
    .description('deploy the specified contracts to specified chain')
    .option('--network [NETWORK]', 'specify which network the contracts will be deployed to')
    .option('--avm [AVM]', 'specify which avm file will be deployed')
    .option('--wallet [WALLET]', 'specify which wallet file will be used')
    .option('--config [CONFIG]', 'specify which deploy config file will be used')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        console.log('Deploying...');
        const deployer = new deployer_1.Deployer();
        await deployer.deploy(projectDir, options.network, options.avm, options.wallet, options.config);
    });
});
program
    .command('invoke')
    .description('invoke the function list in default-config or specify config.')
    .option('--network [NETWORK]', 'specify which network the contracts will be invoked')
    .option('--wallet <WALLET>', 'specify which wallet file will be used')
    .option('--functions <FUNCTIONS>', 'specify which function will be executed')
    .option('--config <CONFIG>', 'specify which config file will be used')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        console.log('Invoking...');
        const invoker = new invoker_1.Invoker();
        await invoker.invoke(projectDir, options.functions, options.network, options.wallet, options.config, program.debug);
    });
});
program
    .command('list')
    .description('list all the function in default-config or specified config.')
    .option('--config <CONFIG>', 'specify which config file will be used')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const invoker = new invoker_1.Invoker();
        const functions = await invoker.list(projectDir, options.config);
        console.log('Functions:');
        functions.forEach((func) => console.log(func));
    });
});
program
    .command('smartx')
    .description('Ontology smart contract IDE - SmartX (http://smartx.ont.io/)')
    .action(() => {
    console.log();
    console.log('Ontology smart contract IDE - SmartX.');
    console.log('Please go to Smartx for debugging smart contracts: http://smartx.ont.io/#/');
    console.log();
});
program
    .command('node')
    .description('Ontology Blockchain private net in test mode')
    .action(() => {
    console.log();
    console.log('Ontology Blockchain private net in test mode.');
    console.log('Please download from: https://github.com/punicasuite/solo-chain/releases');
    console.log();
});
program
    .command('scpm')
    .description('smart contract package manager，support download and publish')
    .action(() => true);
const testCmd = program.command('test');
testCmd.description('test Smart contracts');
testCmd.forwardSubcommands();
testCmd
    .command('template')
    .description('generate test template file')
    .option('--abi <ABI>', 'specify which abi file to be used')
    .option('--config [CONFIG]', 'specify which config file will be used')
    .option('--wallet <WALLET>', 'specify which wallet file will be used')
    .action((options) => {
    const abi = options.abi;
    cliUtils_1.checkRequiredOption('abi', abi);
    const projectDir = cliUtils_1.getProjectDir(program);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        console.log('Generating template from abi file...');
        const tester = new tester_1.Tester();
        tester.template(projectDir, options.config, abi, options.wallet, program.debug);
        console.log('Template ready.');
    });
});
testCmd
    .command('exec')
    .description('execute the test file')
    .option('--file <FILE>', 'specify which test file to execute')
    .action((options) => {
    const file = options.file;
    cliUtils_1.checkRequiredOption('file', file);
    const projectDir = cliUtils_1.getProjectDir(program);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        console.log('Starting test...');
        const tester = new tester_1.Tester();
        await tester.exec(__dirname, projectDir, file);
        console.log('Test complete');
    });
});
const walletCmd = program.command('wallet');
walletCmd.description('manage your ontid, account, asset');
walletCmd.forwardSubcommands();
const accountCmd = walletCmd.command('account');
accountCmd.description('manage your account');
accountCmd.forwardSubcommands();
const assetCmd = walletCmd.command('asset');
assetCmd.description('manager your asset, transfer, balance, withdraw ong, unbound ong');
assetCmd.forwardSubcommands();
accountCmd
    .command('add')
    .description('add account to wallet.json')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        console.log('Creating account...');
        const manager = new walletManager_1.WalletManager();
        manager.init(projectDir, options.wallet, true);
        await manager.addAccount();
        console.log('Adding account successful.');
    });
});
accountCmd
    .command('delete')
    .description('delete account by address.')
    .option('--address <ADDRESS>', 'specify address to delete')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    const address = options.address;
    cliUtils_1.checkRequiredOption('address', address);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const manager = new walletManager_1.WalletManager();
        manager.init(projectDir, options.wallet, false);
        await manager.deleteAccount(address);
        console.log('Deleting account successful.');
    });
});
accountCmd
    .command('import')
    .description('import account by private key.')
    .option('--privateKey <PRIVATE_KEY>', 'specify private key to import in HEX format')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        console.log('Importing account...');
        const manager = new walletManager_1.WalletManager();
        manager.init(projectDir, options.wallet, true);
        await manager.importAccount(options.privateKey);
        console.log('Importing account successful.');
    });
});
accountCmd
    .command('list')
    .description('list all your account address.')
    .option('--wallet [WALLET]', 'specify which wallet file will be used')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const manager = new walletManager_1.WalletManager();
        manager.init(projectDir, options.wallet);
        const accounts = manager.listAccounts();
        console.log('Accounts:');
        accounts.forEach((account) => console.log(account));
    });
});
const ontCmd = walletCmd.command('ontid');
ontCmd.description('manage your identity');
ontCmd.forwardSubcommands();
ontCmd
    .command('add')
    .description('add identity to wallet.json')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        console.log('Creating identity...');
        const manager = new walletManager_1.WalletManager();
        manager.init(projectDir, options.wallet, true);
        await manager.addIdentity();
        console.log('Adding ontid successful.');
    });
});
ontCmd
    .command('delete')
    .description('delete identity by ontid')
    .option('--ontid <ONTID>', 'specify ontid to delete')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    const ontid = options.ontid;
    cliUtils_1.checkRequiredOption('ontid', ontid);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const manager = new walletManager_1.WalletManager();
        manager.init(projectDir, options.wallet, false);
        await manager.deleteIdentity(ontid);
        console.log('Deleting identity successful.');
    });
});
ontCmd
    .command('list')
    .description('list all your identity ontids')
    .option('--wallet [WALLET]', 'specify which wallet file will be used')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const manager = new walletManager_1.WalletManager();
        manager.init(projectDir, options.wallet);
        const identities = manager.listIdentities();
        console.log('Identities:');
        identities.forEach((identity) => console.log(identity));
    });
});
assetCmd
    .command('transfer')
    .description('transfer your asset to another address')
    .option('--asset [ASSET]', 'asset of ONT or ONG (default: "ont")')
    .option('--sender <SENDER>', 'transfer-out account')
    .option('--to <TO>', 'transfer-in account')
    .option('--wallet [WALLET]', 'specify which wallet file will be used')
    .option('--amount <AMOUNT>', 'transfer amount')
    .option('--gas_price [GAS_PRICE]', 'gas price of transaction (default: 0)')
    .option('--gas_limit [GAS_LIMIT]', 'gas limit of the transaction (default: 20000)')
    .option('--network [NETWORK]', 'specify which network will be used (default testNet)')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    const sender = options.sender;
    cliUtils_1.checkRequiredOption('sender', sender);
    const to = options.to;
    cliUtils_1.checkRequiredOption('to', to);
    const amount = options.amount;
    cliUtils_1.checkRequiredOption('amount', amount);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const assets = new assets_1.Assets();
        await assets.transfer(projectDir, options.asset, sender, to, options.wallet, amount, options.gas_price, options.gas_limit, options.network);
    });
});
assetCmd
    .command('withdrawOng')
    .description('withdraw unbound ong')
    .option('--claimer <CLAIMER>', 'transfer-out account')
    .option('--to <TO>', 'transfer-in account')
    .option('--wallet [WALLET]', 'specify which wallet file will be used')
    .option('--amount <AMOUNT>', 'transfer amount')
    .option('--gas_price [GAS_PRICE]', 'gas price of transaction (default: 0)')
    .option('--gas_limit [GAS_LIMIT]', 'gas limit of the transaction (default: 20000)')
    .option('--network [NETWORK]', 'specify which network will be used (default testNet)')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    const claimer = options.claimer;
    cliUtils_1.checkRequiredOption('claimer', claimer);
    const to = options.to;
    cliUtils_1.checkRequiredOption('to', to);
    const amount = options.amount;
    cliUtils_1.checkRequiredOption('amount', amount);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const assets = new assets_1.Assets();
        await assets.withdrawOng(projectDir, claimer, to, options.wallet, amount, options.gas_price, options.gas_limit, options.network);
    });
});
assetCmd
    .command('balanceOf')
    .description('query balance of the address')
    .option('--asset [ASSET]', 'asset of ONT or ONG (default: "ont")')
    .option('--address <ADDRESS>', 'query balance of address')
    .option('--network [NETWORK]', 'specify which network will be used (default testNet)')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    const address = options.address;
    cliUtils_1.checkRequiredOption('address', address);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const assets = new assets_1.Assets();
        const balance = await assets.balanceOf(projectDir, options.asset, address, options.network);
        console.log(`${address} Balance: ${balance}`);
    });
});
assetCmd
    .command('unboundOng')
    .description('query unbound ong')
    .option('--address [ADDRESS]', 'query unbound ong of address')
    .option('--network [NETWORK]', 'specify which network will be used (default testNet)')
    .action((options) => {
    const projectDir = cliUtils_1.getProjectDir(program);
    const address = options.address;
    cliUtils_1.checkRequiredOption('address', address);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const assets = new assets_1.Assets();
        const balance = await assets.unboundOng(projectDir, options.address, options.network);
        console.log(`${address} UnboundOng: ${balance}`);
    });
});
const toolCmd = program.command('tool');
toolCmd.description('tooling functions');
toolCmd.forwardSubcommands();
const transformCmd = toolCmd.command('transform');
transformCmd.description('transform data');
transformCmd.forwardSubcommands();
transformCmd
    .command('addresstohex')
    .description('transform address to hex')
    .option('--input [INPUT]', 'address')
    .action((options) => {
    const input = options.input;
    cliUtils_1.checkRequiredOption('input', input);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const tooler = new tooler_1.Tooler();
        console.log('Result is:');
        console.log(tooler.addressToHex(input));
    });
});
transformCmd
    .command('stringtohex')
    .description('transform string to hex')
    .option('--input [INPUT]', 'arbitrary string')
    .action((options) => {
    const input = options.input;
    cliUtils_1.checkRequiredOption('input', input);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const tooler = new tooler_1.Tooler();
        console.log('Result is:');
        console.log(tooler.str2hex(input));
    });
});
transformCmd
    .command('hexreverse')
    .description('reverse hex string')
    .option('--input [INPUT]', 'arbitrary hex string')
    .action((options) => {
    const input = options.input;
    cliUtils_1.checkRequiredOption('input', input);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const tooler = new tooler_1.Tooler();
        console.log('Result is:');
        console.log(tooler.hexReverse(input));
    });
});
transformCmd
    .command('numtohex')
    .description('transform number to NeoVM hex string')
    .option('--input [INPUT]', 'arbitrary number')
    .action((options) => {
    const input = options.input;
    cliUtils_1.checkRequiredOption('input', input);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const tooler = new tooler_1.Tooler();
        console.log('Result is:');
        console.log(tooler.num2hex(input));
    });
});
transformCmd
    .command('generateprivatekey')
    .description('generate random private key')
    .action((options) => {
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const tooler = new tooler_1.Tooler();
        console.log('Result is:');
        console.log(tooler.randomPrivateKey());
    });
});
toolCmd
    .command('decryptprivatekey')
    .description('decrypt encoded private key')
    .option('--key [KEY]', 'ecnrypted private key')
    .option('--address [ADDRESS]', 'address')
    .option('--salt [SALT]', 'salt')
    .option('--n [N]', 'SCrypt N parameter')
    .option('--password [PASSWORD]', 'password')
    .action((options) => {
    const key = options.key;
    cliUtils_1.checkRequiredOption('key', key);
    const address = options.address;
    cliUtils_1.checkRequiredOption('address', address);
    const salt = options.salt;
    cliUtils_1.checkRequiredOption('salt', salt);
    const n = options.n;
    cliUtils_1.checkRequiredOption('n', n);
    const password = options.password;
    cliUtils_1.checkRequiredOption('password', password);
    return cliUtils_1.wrapDebug(program.debug, async () => {
        const tooler = new tooler_1.Tooler();
        const sk = await tooler.decryptPrivateKey(key, address, salt, n, password);
        console.log('Result is:');
        console.log(sk);
    });
});
program.parse(process.argv);
if (!process.argv.slice(2).length) {
    program.outputHelp();
}
//# sourceMappingURL=punica-ts.js.map